function [Z,P,K,M] = pca(X,n)
% [Z,P,K,M]=PCA(X,n)
%
% Z - Matrix X having zero means.
% P - Eigenvectors(sorted) of matrix Z. Each column represents an eigenvector.
% K - All Eigenvalues (sorted) of matrix Z. It is a column vector.
% M - Grand mean of matrix X (line vector).
%
% X - Matrix containing the train. set, whose each line points to a face data.
% n - Reduction of dimension. The max. n is the number of lines (faces) of X.
% 
% Carlos Thomaz, DoC-IC/London, 17/feb/2001.

%-------------------------------------------------------------------------------
% Validation
%-------------------------------------------------------------------------------

if ((n <= 0) | (n>=size(X,1))), error('Reduction of dimension (n) invalid.'), end

%-------------------------------------------------------------------------------
% Computation
%-------------------------------------------------------------------------------

m = size(X,1);                              % Number of lines or observations
O = ones(size(X,1),1);                      % Matrix all ones
M = mean(X);                                % Mean of each variable
Z = X-(O*M);                               % Transform to zero means variables
V = Z* Z'./(m-1);                          % Inner prod. of X standard (Z) 
[Qv,Qa] = eig(V, 'nobalance');              % Qv(eigenvects), Qa(eigenvals)

Pt = (Z' * Qv * (Qa^(-0.5)))./(sqrt(m-1));    % Eigenvectors of outer prod. of X (Ortonormaliza a matriz)
%-------------------------------------------------------------------------------
% Sort
% Sort in descending order the eigenvalues.
% Eigenvectors whose have the biggest eigenvalues at the beginning of the matrix
%-------------------------------------------------------------------------------

[Ks,Ki] = sort(diag(Qa));
Ki = flipud(Ki);

P  = Pt(:,Ki(1:n));

K  = flipud(Ks);
K  = K(1:n);                             % Return only n eigenvalues

%-------------------------------------------------------------------------------
